'' =================================================================================================
''
''   File....... jm_etimer_demo.spin
''   Purpose.... Background timer
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2009-2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 21 JUL 2009
''   Updated.... 15 JUN 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  et   : "jm_etimer"
  term : "fullduplexserial"


var

  long  tregs                                                   ' address of timer regs
  long  stack[16]

  
pub main

  et.start(16, et#ET_HOLD)                                      ' 1Hz heartbeat on P16
  tregs := et.address                                           ' get address of registers

  cognew(dasblinker(23), @stack)                                ' launch Spin cog
  
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial 
  waitcnt(clkfreq / 100 + cnt)                                  ' wait 10ms
  term.rxflush

  term.tx(CLS)
  term.str(string("Timer Demo - V1.2"))

  et.resume                                                     ' let timer run

  repeat
    term.tx(GOTOXY)
    term.tx(0)
    term.tx(2)

    dec2(long[tregs][et#REG_HR])
    term.tx(":")
    dec2(long[tregs][et#REG_MN])
    term.tx(":")
    dec2(long[tregs][et#REG_SC])
    term.tx(".")
    dec3(long[tregs][et#REG_MS])

    ' press key to hold timer; press againt to resume
    
    if (term.rxcheck => 0)                                      ' key?
      if (et.running)                                           ' if running
        et.hold                                                 '   hold timer
      else                                                      ' else
        et.resume                                               '   resume
      term.rxflush                                              ' flush kbd buffer
    else
      pause(2)                                                  ' prevent terminal overrun   


pub dec2(n)

  if (n < 10)
    term.tx("0")
    
  term.dec(n)


pub dec3(n)

  if (n < 10)
    term.tx("0")

  if (n < 100)
    term.tx("0")
    
  term.dec(n)


pri dasblinker(pin) | s                                         ' launch with cognew

'' pin will be high when 1s digit of seconds reg is 5

  dira[pin] := 1
  outa[pin] := 0

  repeat
    s := long[tregs][et#REG_SC] // 10                           ' isolate 1s digit
    outa[pin] := (s == 5)                                       ' on when digit is 5

    if (long[tregs][et#REG_MN] == 1)                            ' one minute elapsed?
      et.reset                                                  ' yes, reset timer


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  